@@ -24,8 +24,6 @@
 analyzeFrame = function(frame, bin, freqs,
                         autoCorrelation = NULL,
                         samplingRate,
-                        scaleCorrection,
-                        loudness,
                         cutFreq,
                         trackPitch = TRUE,
                         pitchMethods = c('dom', 'autocor'),
@@ -58,13 +56,11 @@ analyzeFrame = function(frame, bin, freqs,
   absSpec_cut$w = absSpec_cut$amp / amplitude
   specCentroid = sum(absSpec_cut$freq * absSpec_cut$w)
   peakFreq = absSpec_cut$freq[which.max(absSpec_cut$amp)]
+
+  # quartiles of spectral energy distribution
   cums = cumsum(absSpec_cut$amp)
-  # first quartile of spectral energy distribution
   quartile25 = absSpec_cut$freq[min(which(cums >= 0.25 * amplitude))]
-  # second quartile (same as medianFreq within this spectral band)
   quartile50 = absSpec_cut$freq[min(which(cums >= 0.5 * amplitude))]
-  # third quartile. Note: half the energy in the band from pitchFloor to
-  # cutFreq kHz lies between quartile25 and quartile75
   quartile75 = absSpec_cut$freq[min(which(cums >= 0.75 * amplitude))]
 
   # get spectral slope in dB/kHz
@@ -73,22 +69,11 @@ analyzeFrame = function(frame, bin, freqs,
   # plot(absSpec_cut$freq, absSpec_cut$amp_dB, type = 'l')
   specSlope = summary(lm(amp_dB ~ freq, data = absSpec_cut))$coef[2, 1] * 1000
 
-  # scale correction for loudness estimation
-  if (is.numeric(scaleCorrection)) {
-    loudness = getLoudnessPerFrame(
-      spec = frame * scaleCorrection,
-      samplingRate = samplingRate,
-      spreadSpectrum = loudness$spreadSpectrum
-    )  # in sone, assuming scaling by SPL_measured in analyze()
-  } else {
-    loudness = NA
-  }
-
   ## PITCH TRACKING
   frame = frame / max(frame) # plot(frame, type='l')
 
   # lowest dominant frequency band
-  if (trackPitch & 'dom' %in% pitchMethods) {
+  if (trackPitch & any(pitchMethods == 'dom')) {
     d = do.call(getDom,
                 c(pitchDom,
                   list(frame = frame,
@@ -188,10 +173,9 @@ analyzeFrame = function(frame, bin, freqs,
     # plot (a, b, ylim = c(0, 1))
   }
 
-  return(list(
+  list(
     'pitchCands_frame' = pitchCands_frame,
     'summaries' = data.frame(
-      loudness = loudness,
       HNR = HNR,
       dom = dom,
       specCentroid = specCentroid,
@@ -201,9 +185,10 @@ analyzeFrame = function(frame, bin, freqs,
       quartile75 = quartile75,
       specSlope = specSlope
     )
-  ))
+  )
 }
 
+
 #' Get prior for pitch candidates
 #'
 #' Prior for adjusting the estimated pitch certainties in \code{\link{analyze}}.
@@ -300,6 +285,7 @@ getPrior = function(priorMean,
   }
 }
 
+
 #' Summarize the output of analyze()
 #'
 #' Internal soundgen function
@@ -323,14 +309,14 @@ summarizeAnalyze = function(
 
   # pre-parse summary function names to speed things up
   functions = vector('list', length(summaryFun))
-  for (f in 1:length(summaryFun)) {
+  for (f in seq_along(summaryFun)) {
     functions[[f]] = eval(parse(text = summaryFun[f]))
   }
 
   # apply the specified summary function to each column of result
   out = list()
   for (v in vars) {
-    for (s in 1:ls) {
+    for (s in seq_len(ls)) {
       # remove NAs for the most common summary functions
       if (summaryFun[s] %in% c('mean', 'median', 'sd', 'min', 'max', 'range', 'sum')) {
         var_values = na.omit(result[, v])
@@ -363,7 +349,7 @@ summarizeAnalyze = function(
     out = c(temp, out)
   }
 
-  return(as.data.frame(out))
+  as.data.frame(out)
 }
 
 
@@ -441,7 +427,7 @@ updateAnalyze = function(
 
     # Calculate subharmonics-to-harmonics ratio
     for (f in voiced_frames) {
-      temp = try(do.call('subhToHarm', c(
+      temp = try(do.call('getSHR', c(
         subh_pars,
         list(frame = spectrogram[, f],
              bin = bin,
@@ -530,8 +516,7 @@ updateAnalyze = function(
   # plot(result$time, result$fmDep, type = 'b', cex = result$fmPurity * 10)
 
   # Arrange columns in alphabetical order (except the first three)
-  result = result[, c(1:3, 3 + order(colnames(result)[4:ncol(result)]))]
-  return(result)
+  result[, c(1:3, 3 + order(colnames(result)[4:ncol(result)]))]
 }
 
 
@@ -567,7 +552,7 @@ formatPitchManual = function(pitchManual) {
         # file OK
         pitchManual_list = vector('list', nrow(pitchManual_df))
         names(pitchManual_list) = pitchManual_df$file
-        for (i in 1:nrow(pitchManual_df)) {
+        for (i in seq_len(nrow(pitchManual_df))) {
           pitchManual_list[[i]] = suppressWarnings(as.numeric(unlist(strsplit(
             as.character(pitchManual_df$pitch[[i]]), ','))))
         }
@@ -591,7 +576,7 @@ formatPitchManual = function(pitchManual) {
       pitchManual_df = as.data.frame(pitchManual)
       pitchManual_list = vector('list', nrow(pitchManual_df))
       names(pitchManual_list) = pitchManual_df$file
-      for (i in 1:nrow(pitchManual_df)) {
+      for (i in seq_len(nrow(pitchManual_df))) {
         pitchManual_list[[i]] = suppressWarnings(as.numeric(unlist(strsplit(
           as.character(pitchManual_df$pitch[[i]]), ','))))
       }
@@ -613,258 +598,5 @@ formatPitchManual = function(pitchManual) {
       "'file' and 'pitch', or a named list with pitch contours per file"))
   }
 
-  return(pitchManual_list)
-}
-
-
-#' Get flux from features
-#'
-#' Internal soundgen function
-#'
-#' Calculates the change in acoustic features returned by analyze() from one
-#' STFT frame to the next. Since the features are on different scales, they are
-#' normalized depending on their units (but not scaled). Flux is calculated as
-#' mean absolute change across all normalized features. Whenever flux exceeds
-#' \code{thres}, a new epoch begins.
-#' @param an dataframe of results from analyze()
-#' @param thres threshold used for epoch detection (0 - 1)
-#' @param smoothing_ww if > 1, \code{\link{medianSmoother}} is called on input dataframe
-#' @param plot if TRUE, plots the normalized feature matrix and epochs
-#' @return Returns a data frame with flux per frame and epoch numbers.
-#' @keywords internal
-#' @examples
-#' an = analyze(soundgen(), 16000)
-#' fl = soundgen:::getFeatureFlux(an$detailed, plot = TRUE)
-#' \dontrun{
-#' # or simply:
-#' an = analyze(soundgen(sylLen = 500), 16000, plot = TRUE, ylim = c(0, 8),
-#'              extraContour = 'flux', flux = list(smoothWin = 100, thres = .15))
-#' }
-getFeatureFlux = function(an,
-                          thres = 0.1,
-                          smoothing_ww = 1,
-                          plot = FALSE) {
-  if (nrow(an) == 1) return(data.frame(frame = 1, flux = 0, epoch = 1))
-  # just work with certain "trustworthy" variables listed in soundgen:::featureFlux_vars
-  m = an[, match(featureFlux_vars$feature, colnames(an))]
-
-  # remove columns with nothing but NAs
-  col_rm = which(apply(m, 2, function(x) !any(!is.na(x))))
-  if (length(col_rm) > 0) m = m[, -col_rm]
-
-  # log-transform features measured in Hz
-  for (i in 1:ncol(m)) {
-    if (featureFlux_vars$log_transform[i]) {
-      m[, i] = log2(m[, i] + 1)  # +1 b/c otherwise 0 produces NA
-    }
-  }
-
-  # normalize according to unit of measurement (don't z-transform because then
-  # even uniform files will show spurious variation - the changes here should be
-  # absolute, not relative)
-  cm = colMeans(m, na.rm = TRUE)
-  cm[which(colnames(m) == 'voiced')] = 0  # voiced
-  for (i in 1:ncol(m)) {
-    # if (featureFlux_vars$feature[i] != 'voiced')
-    m[, i] = (m[, i] - cm[i]) / featureFlux_vars$norm_scale[i]
-  }
-  # m[is.na(m)] = 0   # NAs become 0 (mean)
-  m$voiced = as.numeric(m$voiced)
-  # summary(m)
-
-  # median smoothing
-  if (smoothing_ww > 1) {
-    m = medianSmoother(m, smoothing_ww = smoothing_ww, smoothingThres = 0)
-  }
-
-  # calculate the average change from one STFT frame to the next and segment into epochs
-  nFrames = nrow(m)
-  flux = rep(NA, nFrames)
-  epoch = rep(1, nFrames)
-  for (i in 2:nFrames) {
-    cor_i = cor(as.numeric(m[i, ]), as.numeric(m[i - 1, ]), use = 'complete.obs')
-    flux[i] = 1 - (cor_i + 1) / 2  # cor_i = -1 gives a flux of 1, 0 -> 0.5, 1 -> 1
-    if (is.finite(flux[i]) && flux[i] > thres) {
-      epoch[i] = epoch[i - 1] + 1
-    } else {
-      epoch[i] = epoch[i - 1]
-    }
-  }
-
-  # plotting
-  if (plot) {
-    transitions = which(diff(epoch) != 0) - 0.5
-    image(as.matrix(m))
-    points(seq(0, 1, length.out = length(flux)), flux, type = 'l')
-    if (length(transitions) > 0) {
-      for (t in transitions) abline(v = t / nFrames)
-    }
-  }
-  return(data.frame(frame = 1:nFrames, flux = flux, epoch = epoch))
-}
-
-
-#' Get spectral flux
-#'
-#' Internal soundgen function
-#'
-#' Calculates spectral flux: the average change across all spectral bins from
-#' one STFT frame to the next. Spectra are normalized in each frame, so
-#' amplitude changes have no effect on flux.
-#' @return vector of length ncol(s)
-#' @param s raw spectrogram (not normalized): rows = frequency bins, columns = STFT frames
-#' @keywords internal
-getSpectralFlux = function(s) {
-  nc = ncol(s)
-  for (c in 1:nc) s[, c] = s[, c] / max(s[, c])  # normalize
-  flux = rep(0, nc)
-  for (c in 2:nc) flux[c] = mean(abs(s[, c] - s[, c - 1]))
-  # or as.numeric(dist(rbind(s[, c], s[, c - 1])))
-  return(flux)
-}
-
-
-#' Get HNR
-#'
-#' Calculates the harmonics-to-noise ratio (HNR) - that is, the ratio between
-#' the intensity (root mean square amplitude) of the harmonic component and the
-#' intensity of the noise component.
-#'
-#' @param x time series (a numeric vector)
-#' @param samplingRate sampling rate
-#' @param acf_x pre-computed autocorrelation function of input \code{x}
-#' @param lag.min,lag.max minimum and maximum lag to consider when looking for
-#'   peaks in the ACF
-#' @param interpol method of improving the frequency resolution by interpolating
-#'   the ACF: "none" = don't interpolate; "parab" = parabolic interpolation on
-#'   three points (local peak and its neighbors); "spline" = spline
-#'   interpolation; "sinc" = sin(x)/x interpolation to a continuous function
-#'   followed by a search for local peaks using Brent's method
-#' @param wn window applied to \code{x} (unless acf_x is provided instead of x)
-#'   as well as to the sinc interpolation
-#' @param idx_max (interal) the index of the peak to investigate, if already
-#'   estimated
-#' @keywords internal
-#'
-#' @examples
-#' signal = sin(2 * pi * 150 * (1:16000)/16000)
-#' signal = signal / sqrt(mean(signal^2))
-#' noise = rnorm(16000)
-#' noise = noise / sqrt(mean(noise^2))
-#' SNR = 40
-#' s = signal + noise * 10^(-SNR/20)
-#' soundgen:::getHNR(s, 16000, lag.min = 16000/1000,
-#' lag.max = 16000/75, interpol = 'none')
-#' soundgen:::getHNR(s, 16000, lag.min = 16000/1000,
-#' lag.max = 16000/75, interpol = 'parab')
-#' soundgen:::getHNR(s, 16000, lag.min = 16000/1000,
-#' lag.max = 16000/75, interpol = 'spline')
-#' soundgen:::getHNR(s, 16000, lag.min = 16000/1000,
-#' lag.max = 16000/75, interpol = 'sinc')
-getHNR = function(x = NULL,
-                  samplingRate = NA,
-                  acf_x = NULL,
-                  lag.min = 2,
-                  lag.max = length(x),
-                  interpol = c('none', 'parab', 'spline', 'sinc')[4],
-                  wn = 'hanning',
-                  idx_max = NULL
-) {
-  if (!is.null(x)) {
-    ## calculate ACF
-    len = length(x)
-    half_len = floor(len / 2)
-    lag.min = round(lag.min)
-    lag.max = round(min(lag.max, half_len))
-    if (lag.min > half_len) stop('lag.min is too small')
-    if (lag.max <= lag.min) stop('lag.max must be > lag.min')
-
-    # prepare a windowing function
-    win = seewave::ftwindow(len, wn = wn)
-
-    # calculate ACF of the windowing function
-    sp_win = fft(win) / half_len
-    powerSpectrum_win = Re(sp_win * Conj(sp_win))
-    acf_win = Re(fft(powerSpectrum_win, inverse = TRUE))[1:half_len]
-    acf_win = acf_win / acf_win[1]
-    # plot(acf_win, type = 'l')
-
-    ## calculate ACF of the windowed signal
-    sp_x = fft(x * win) / half_len
-    powerSpectrum_x = Re(sp_x * Conj(sp_x))
-    acf_x = Re(fft(powerSpectrum_x, inverse = TRUE))[1:half_len] / acf_win
-    acf_x = acf_x / acf_x[1]
-  } else {
-    if (is.null(acf_x))
-      stop('Please provide either signal (x) or its autocorrelation function (acf_x)')
-  }
-  # plot(acf_x[lag.min:lag.max], type = 'b')
-
-  ## find the maximum and interpolate the ACF to improve resolution
-  if (is.null(idx_max)) {
-    idx_max = which.max(acf_x[lag.min:lag.max]) + lag.min - 1
-  }
-  # if (acf_x[idx_max] < 0) return(NA)  # causes problems in getPitchAutocor()
-  if (interpol == 'none') {
-    max_acf = acf_x[idx_max]
-  } else if (interpol == 'parab') {
-    parabInterp = parabPeakInterpol(acf_x[(idx_max - 1) : (idx_max + 1)])
-    max_acf = parabInterp$ampl_p
-    idx_max = idx_max + parabInterp$p
-  } else if (interpol == 'spline') {
-    idx = max(lag.min, (idx_max - 10)) : min(lag.max, (idx_max + 10))
-    acf_ups = spline(acf_x[idx], n = length(idx) * 100)
-    idx_max_ups = which.max(acf_ups$y)
-    max_acf = acf_ups$y[idx_max_ups]
-    idx_max = idx[1] - 1 + acf_ups$x[idx_max_ups]
-  } else if (interpol == 'sinc') {
-    # apply a gaussian window to the sinc interpolation as a function of the
-    # distance from the max
-    half_len = min(250, length(acf_x) / 2)
-    idx_left = max(2, idx_max - half_len)
-    # max(2, idx_max - min(250, floor(length(acf_x) / 4)))
-    if (idx_max > idx_left) {
-      win_left = seewave::ftwindow((idx_max - idx_left) * 2, wn = wn)[1:(idx_max - idx_left)]
-    } else {
-      win_left = numeric(0)
-    }
-    idx_right = min(length(acf_x), idx_max + half_len)
-    #   min(length(acf_x), idx_max + min(250, floor(length(acf_x) / 4)))
-    if (idx_right > idx_max) {
-      win_right = seewave::ftwindow((idx_right - idx_max) * 2, wn = wn)[(idx_right - idx_max):((idx_right - idx_max) * 2)]
-    } else {
-      win_right = numeric(0)
-    }
-    win = c(win_left, win_right)
-    # win = win / sum(win)
-    # plot(win)
-    acf_idx = idx_left:idx_right
-    # length(win)
-    # length(acf_idx)
-
-    if (FALSE) {
-      # visual check of sinc interpolation
-      d_new = data.frame(x = seq(lag.min, lag.max, by = .1))
-      for (i in 1:nrow(d_new)) {
-        d_new$y[i] = sum(acf_x[acf_idx] * sinc(d_new$x[i] - acf_idx) * win)
-      }
-      plot(lag.min:lag.max, acf_x[lag.min:lag.max])
-      points(d_new, type = 'l', col = 'blue')
-    }
-
-    # find max by using the sinc function in optimize (Brent 1973)
-    opt = optimize(function(j) {sum(acf_x[acf_idx] * sinc(j - acf_idx) * win)},
-                   interval = c(idx_max - 2, idx_max + 2), maximum = TRUE)
-    # opt = optimize(function(j) {sum(acf_x[acf_idx] * sinc(j - acf_idx) * win)},
-    #                interval = c(lag.min, lag.max), maximum = TRUE)
-    max_acf = opt$objective
-    idx_max = opt$maximum
-  }
-  if (max_acf > 1) {
-    max_acf = 1 / max_acf
-  }
-  f0 = samplingRate / idx_max
-  return(list(f0 = f0,
-              max_acf = max_acf,
-              HNR = to_dB(max_acf)))
+  pitchManual_list
 }
